set linesize 160
set heading  off
set serveroutput on size 800000
set feedback off
set verify off
set echo off
set trimspool on

spool check_oracle.txt

/*
Check dictionary privilege
*/
WHENEVER SQLERROR EXIT 
declare 
    dict_priv integer;
begin
    select count(*) into dict_priv
    from session_privs
    where privilege='SELECT ANY DICTIONARY';

    if dict_priv <> 1 then
         RAISE_APPLICATION_ERROR(-20000, 'User must have SELECT ANY DICTIONARY privilege');
    end if;
end;
/

declare 
    sessions integer;
    ctm_sessions integer;

    value varchar2(600);
    block_size  varchar2(600);   
    os_type char(1);

    system_ts_name varchar2(30);
    system_temp_name varchar2(30);

    cursor schema_list is
    select owner, sum(bytes)/1024/1024 from dba_segments
    group by owner; 
    owner varchar2(30);
    schema_size integer; 

    cursor archive_cursor is
    select value from v$parameter
    where name like 'log_archive_dest__' 
    or name='log_archive_dest'
    or name='log_archive_dest_10'
    or name='standby_archive_dest'
    or name='log_archive_duplex_dest';  

    archive_dest varchar2(30);
    int_db_version integer;
    db_version char(8);
    tmp_name varchar2(30);
    ts_name varchar2(30);
    archive_status varchar2(8);
    edition varchar2(100);
    characterset varchar2(20);
    data_ts_path varchar2(600); 
    indx_ts_path varchar2(600); 
    data_ts_size integer;
    indx_ts_size integer; 

    /* Memory */
    sga_target integer;
    memory_target integer;
    shared_pool_size integer;
    db_cache_size integer;
   
    procedure ex_print (name varchar2, var_val varchar2 default '',linein integer default 0) is 
        printed_text varchar2(600);
    begin 
        if linein = 1 then
    	    dbms_output.put_line('.  ');
    	    dbms_output.put_line('.  ');
        end if;
    	printed_text :=  rpad(name,40) || var_val;
    	dbms_output.put_line(printed_text);
        if linein = 1 then
    	    dbms_output.put_line(substr('=========================',1,length(name)));
        end if;
    end;


    function session_list (ctm_em varchar2) return varchar
    is 
      tabname varchar2(600);
      Cursor c100 is
        select username u,count(*) c
        from v$session
        where username is not null
        and username!='SYSTEM'
        and username in ( select owner from dba_tables where table_name = tabname )
        group by username;
      rec_c100 c100%ROWTYPE;
      value varchar2(600);
    begin
      if ctm_em='CTM' then
	tabname:='CMR_AJF';
      else
	tabname:='ALARM';
      end if;
      value:=NULL;
      open c100;
      LOOP
        fetch c100 into rec_c100;
        exit when c100%NOTFOUND;
        value := value || case when value is null then '' else ',' end || rec_c100.u || '(' || rec_c100.c || ')';
      END LOOP;
      close c100;
      return value;
    end;


begin
/* 
GENERAL
*/
    ex_print('DB General config','',1 );

    select banner into edition
    from v$version
    where rownum=1;

    if INSTRC(edition,'Edition',1,1)=0 then 
       select REPLACE (edition ,'Release', 'Standard Edition Release') into edition from dual;
    end if;

    dbms_output.put_line(edition);
    dbms_output.put_line('-------------------------------------------------------------------');

    select PLATFORM_NAME into value from v$database;
    ex_print('Host OS', value);

    select substr(version,1,8)
    into db_version
    from product_component_version
    where product like 'Oracle%';
   
    select to_number(substr(db_version,1,1)) into int_db_version from dual;
    if int_db_version=1 then
	int_db_version:=10;
    else
	db_version := substr(db_version,1,7);
    end if;

    ex_print('DB_version', db_version );

    if  (substr(db_version,1,2)=10 and substr(db_version,4,1)<2) then 
        dbms_output.put_line('...Warning: BMC recomends using 10.2 and above version');
    elsif (substr(db_version,1,2))<10 then 
        dbms_output.put_line('...Error: BMC recomends using 10.2 and above version');
    end if;

    select instance_name into value 
    from v$instance;
    ex_print('InstanceName',value);

    select host_name into value 
    from v$instance;
    ex_print('Host name',value);

    select SYS_CONTEXT('USERENV','OS_USER') into value  from dual;
    ex_print('OS user who runs the DB ',value);

    select archiver into archive_status 
    from v$instance; 
    ex_print('Archive status',archive_status);

    if archive_status='STARTED' then
       ex_print('Archive destination list','');
       open archive_cursor;
       loop
              fetch archive_cursor into  value;
              exit when archive_cursor%notfound;
              if value is not null then
                     ex_print('...',value);
              end if;
       end loop;
       close archive_cursor; 
    end if; 

    select value into characterset 
    from nls_database_parameters
    where parameter='NLS_CHARACTERSET';

    ex_print('Character set',characterset);

    if (characterset != 'WE8ISO8859P1' and characterset !='US7ASCII' and characterset not like '%UTF8%') then 
        dbms_output.put_line('...Error: '||characterset|| ' is unsupported characterset');
    end if;

/*
Usage
*/
    ex_print('EM and CTM schemas used on this instance ','',1 );

   /* Number of Control-M served */
    select count(*) into value 
    from dba_tables where table_name='CMR_AJF';
    ex_print('Number of Control-M schemas on this DB',value);

   /* Number of EM served */
    select count(*) into value 
    from dba_tables where table_name='ALARM';
    ex_print('Number of EM schemas on this DB',value);

   /* List Large Schema's */
    select 0 into value from dual;
    open schema_list;
    loop
           fetch schema_list into owner , schema_size;
           exit when schema_list%notfound;
           if schema_size > 1023 then
                  if value=0 then 
                     ex_print('List of large schemas','');
                     select 1 into value from dual;
                  end if;
                  ex_print('...',owner||'  '||schema_size||'MB');  
           end if;
    end loop;
    close schema_list;

/*  
Storage Sizes
*/
    ex_print('Verify minimal sizes for storage','',1 );

/* SYSTEM Tablespace size */
    select round(sum(bytes/(1024*1024))) into value
    from dba_data_files 
    where tablespace_name = 'SYSTEM';
    ex_print('SYSTEM tablespace size',value||'MB');
    if value < 500 then
        dbms_output.put_line('...Warning: SYSTEM tablespace should be at least 500MB.');
    end if;
     
/* UNDO-Rollback size */
    begin
        select round(sum(bytes/(1024*1024))) into value
        from dba_data_files
        where tablespace_name = (select  value from v$parameter where name='undo_tablespace'); 

    exception
        when NO_DATA_FOUND then
            value := 0;
        when others then
            raise;
    end;
    ex_print('Undo tablespace size',value||'MB');
    if value <= 100 then
        dbms_output.put_line('...Error: UNDO tablespace should be at least 500MB.'); 
    elsif value < 500 then
        dbms_output.put_line('...Warning: UNDO tablespace should be at least 500MB.');
    end if;

/* TEMP size */
    select round(sum(bytes/1024/1024))
    into value
    from dba_temp_files;
    ex_print('Total TEMP size',value||'MB');
    if value < 300 then
        dbms_output.put_line('...Warning: TEMP tablespace should be at least 300MB.');
    end if;

    select count(*) into value from v$log;
    ex_print('Number of REDO groups',value);
    if value<3 then
	dbms_output.put_line('...Warning: It is recommended that at least 3 REDO LOG groups are used.');
    end if;

    select min(bytes/1024/1024) into value from v$log;
    ex_print('Minimal REDO LOG size',value);
    if value <= 70 then
        dbms_output.put_line('...Error: REDO LOG files should not be smaller than 150MB.');
    elsif value < 150 then
	dbms_output.put_line('...Warning: REDO LOG files should not be smaller than 150MB.');
    end if;


/* 
MEMORY 
*/
    ex_print('Verify minimal values for oracle settings','',1 );

    if (substr(db_version,1,2))<10 then
       sga_target :=0;
    elsif (substr(db_version,1,2))>10 then
       select round(value/1024/1024)
       into memory_target
       from v$parameter
       where name = 'memory_target';

       select round(value/1024/1024)
       into sga_target
       from v$parameter
       where name = 'sga_target';

       select round(value/1024/1024)
       into value
       from v$parameter
       where name = 'memory_max_target';
       ex_print('MEMORY_MAX_TARGET',value||'MB');

    else
       select round(value/1024/1024)
       into sga_target
       from v$parameter
       where name = 'sga_target';

       select round(value/1024/1024)
       into value
       from v$parameter
       where name = 'sga_max_size';
       ex_print('SGA_MAX_SIZE',value||'MB');

    end if;

    select value/1024/1024
    into db_cache_size
    from v$parameter
    where name='db_cache_size';

    select (value/1024/1024)
    into shared_pool_size
    from v$parameter
    where name='shared_pool_size';


    if sga_target!=0 or memory_target!=0  then
        if memory_target!=0 and (substr(db_version,1,2))>10  then
          ex_print('SGA management','AUTO');
          ex_print('MEMORY_TARGET',memory_target||'MB');
          if memory_target<900 then
              dbms_output.put_line('...Warning: MEMORY_TARGET should be at least 900MB.');
              sga_target:=memory_target;
          end if;
       else
          ex_print('SGA management','AUTO');
          ex_print('SGA_TARGET',sga_target||'MB');
          if sga_target<900 then
              dbms_output.put_line('...Warning: SGA_TARGET should be at least 900MB.');
          end if;
       end if;
       
       ex_print('DB_CACHE_SIZE',db_cache_size||'MB');
       if db_cache_size!=0 then 
           if sga_target/db_cache_size<2 then 
                 dbms_output.put_line('...Warning: DB_CACHE_SIZE should not be more than half of the total SGA');
           end if;
       end if;

       ex_print('SHARED_POOL_SIZE',shared_pool_size||'MB');
       if shared_pool_size!=0 then
           if sga_target/shared_pool_size<4 then 
                 dbms_output.put_line('...Warning: SHARED_POOL_SIZE should not be more than quarter of the total SGA');
           end if;
       end if;

    else
       ex_print('SGA management','MANUAL');

       ex_print('DB_CACHE_SIZE',db_cache_size||'MB');
       if db_cache_size<400 then
           dbms_output.put_line('...Error: DB_CACHE_SIZE should not be smaller than 500MB'); 
       elsif db_cache_size<500 then 
          dbms_output.put_line('...Warning: DB_CACHE_SIZE should not be smaller than 500MB'); 
       end if;
     
       ex_print('SHARED_POOL_SIZE',shared_pool_size||'MB');
       if shared_pool_size<150 then
          dbms_output.put_line('...Error: SHARED_POOL_SIZE should not be smaller than 200MB');
       elsif shared_pool_size<200 then  
          dbms_output.put_line('...Warning: SHARED_POOL_SIZE should not be smaller than 200MB');
       end if;

    end if;

    select value
    into value 
    from v$parameter
    where name = 'workarea_size_policy';

    if value='AUTO' then
         if memory_target=0  or (substr(db_version,1,2))<11 then  
           select round(value/1024/1024) 
           into value
           from v$parameter 
           where name = 'pga_aggregate_target';
           ex_print('PGA_AGGREGATE_TARGET',value||'MB');
           if value<100 then
              dbms_output.put_line('...Error: PGA_AGGREGATE_TARGET should not be smaller than 200MB'); 
           elsif value<200 then
              dbms_output.put_line('...Warning: PGA_AGGREGATE_TARGET should not be smaller than 200MB');
           end if;
       end if;
    else
       select value/1024/1024
       into value
       from v$parameter
       where name = 'sort_area_size';
       ex_print('SORT_AREA_SIZE',value||'MB');
       if value<2 then 
         dbms_output.put_line('...Warning: SORT_AREA_SIZE should not be smaller than 2MB');  
       end if;
    end if;

    select value into value
    from v$parameter where name = 'processes';
    ex_print('PROCESSES',value);
    if value<300 then
         dbms_output.put_line('...Warning: PROCESSES should not be smaller than 300');
    end if;

    select value into value
    from v$parameter where name = 'log_buffer';
    ex_print('LOG_BUFFER',value||' Bytes');
    if value<250000 then
         dbms_output.put_line('...Warning: LOG_BUFFER should not be smaller than 250000 Bytes');
    end if;


    ex_print('Verify minimal values for oracle settings','',1 );

    select value into block_size
    from v$parameter where name = 'db_block_size';
    ex_print('DB_BLOCK_SIZE',block_size);
    if block_size<8192 then 
         dbms_output.put_line('...Warning: DB_BLOCK_SIZE smaller than 8192 may decrease performance '); 
    end if;

    select value into value
    from v$parameter where name = 'optimizer_mode';
    ex_print('OPTIMIZER_MODE',value);
    if upper(value)!='ALL_ROWS' and upper(value)!='CHOOSE' then
         dbms_output.put_line('...Warning: OPTIMIZER_MODE preferable  value is ALL_ROWS ');
    end if;

    select value into value
    from v$parameter where name = 'cursor_sharing';
    ex_print('CURSOR_SHARING',value);
    if upper(value)!='SIMILAR' then
         dbms_output.put_line('...Warning: CURSOR_SHARING preferable  value is SIMILAR ');
    end if;

/*
RAC SETTINGS
*/

    ex_print('Check RAC parameters','',1 );

    select value into value
    from v$parameter where name = 'cluster_database';
    ex_print('RAC',value);

    select value into value
    from v$parameter where name = 'max_commit_propagation_delay';
    ex_print('MAX_COMMIT_PRPAGATION_DELAY',value);

end;
/

exit
