#!/bin/sh
#
#  Script %name:	checkReqRun.sh %
#  Instance:		1
#  %version:		4 %
#  Description:		
#  %created_by:		psimonov %
#  %date_created:	Wed Apr 15 17:23:19 2015 %
#  %modified_by		akarmon %
#  %date_modified:  Mon Dec 18 14:00:00 2017 %

[ "${generate_debug}" = "Y" ] && set -x




# get file/directory name and return it as absoulte path
fix_path()
{
    [ "${generate_debug}" = "Y" ] && set -x

    base_name=`basename "${1}"` # script name
    dir_name=`dirname "${1}"`   # directory path

    [ "${dir_name}" = "." ] && dir_name=`pwd`

# make sure directory path is absoulte
    first_char=`echo "${dir_name}" | cut -c1`
    [ "${first_char}" = "/" ] || dir_name=`pwd`/${dir_name}
    [ "${dir_name}"   = "/" ] && dir_name='' # avoid the extra '/'

    echo "${dir_name}/${base_name}"
}

usage()
{
	echo "Usage:"
	echo "checkReqRun.sh -product=<PRODUCT_NAME> -memory_model=<MEMORY_MODEL> -database=<DATABASE> -BMC_INST_JAVA_HOME=<java location>"
	echo "<PRODUCT_NAME> valid values: CTMS/EM/ONE/AGENT/ARCHIVE"
	echo "<MEMORY_MODEL> valid values: SMALL/MEDIUM/LARGE/NONE"
	echo "<database> valid values: ORACLE/PostgreSQL"
	exit 1
}


handle_startup_args()
{
    [ "${generate_debug}" = "Y" ] && set -x

    # maven_args="-o"
    # profiles=""
    # clean_step="clean"

# parse scripts arguments, ignore undefined option
    for option in ${started_args}
    do
	var_name=`echo "${option}"  | cut -d= -f1`
	var_value=`echo "${option}" | cut -d= -f2`

	case ${var_name} in
	    '-product')
		product="${var_value}"
		;;
	    '-version')
		version="${var_value}"
		;;
	    '-memory_model')
		memory_model="${var_value}"
		;;
	    '-database')
		database="${var_value}"
		database=`echo ${database} | tr '[a-z]' '[A-Z]'`
		if [ "${database:-""}" = "POSTGRESQL" ]; then
			database="PostgreSQL"
		fi
		;;
	    '-BMC_INST_JAVA_HOME')
		JAVA_LOCATION="${var_value}"
		USER_SUPPLIED_JRE="Y"
		;;
		*)
		usage;;
	esac
    done

    
    # if [ -n "$profiles" ]; then
		# maven_args="${maven_args} -P${profiles}"
    # fi

    # if [ -z "${run_tests}" ]; then 
		# maven_args="${maven_args} -Dmaven.test.skip=true"
	# else
		# maven_args="${maven_args} -Dtest=${run_tests}"
    # fi
}

# return os major version
get_os_version()
{
	suse_flavor="n"			# SuSE Linux
	redhat_flavor="n" 		# Red Hat, CentOS, Alma and Amazon Linux
	ubuntu_flavor="n"		# Ubuntu Linux
	oracle_flavor="n"		# Oracle Linux
	
	os_major_version=""
	
	machine_os_type=`uname`	
	
	uname -a | grep 'HP-UX' > /dev/null 2>&1
	if [ $? -eq 0 ]; then
			echo "HP platform is not supported."
			exit 1
	fi
	uname -a | grep 's390' > /dev/null 2>&1
	if [ $? -eq 0 ]; then
			echo "Check_req is not supported on s390"
			exit 1
	fi
	
	case ${machine_os_type} in
		linux|Linux)
		######### Red Hat, CentOS , Alma #########
			if [ -f "/etc/redhat-release" ]; then
				redhat_flavor="y"	
				os_major_version=5	# Setting default
				versionToken=`cat /etc/redhat-release | grep release | awk '{for ( i=0; i<=NF; i ++)  if($i~/[0-9]+\.*[0-9]*/) ver=$i }; END {print ver}'`
				if [ -n "${versionToken}" ]; then
					os_major_version=`echo $versionToken |  sed -e 's/\.[0-9]*//g'`
				fi
	
		######### Amazon Linux #########
			elif [ -f "/etc/system-release" ]; then
				redhat_flavor="y"	
				os_major_version=2015	# Setting default
				versionToken=`cat /etc/system-release | grep release | awk '{for ( i=0; i<=NF; i ++)  if($i~/[0-9]+\.*[0-9]*/) ver=$i }; END {print ver}'`
				if [ -n "${versionToken}" ]; then
					os_major_version=`echo $versionToken |  sed -e 's/\.[0-9]*//g'`
				fi
				
		######### SuSE Linux #########
			elif [ -f "/etc/SuSE-release" ]; then
				suse_flavor="y"
				os_major_version=`cat /etc/SuSE-release | grep VERSION | awk '{for ( i=0; i<=NF; i ++)  if($i~/[0-9]+\.*[0-9]*/) ver=$i }; END {print ver}'`
		
		######### Ubuntu Linux #########
			elif [ -f "/etc/lsb-release" ]; then
				ubuntu_flavor="y"
				versionToken=`cat /etc/lsb-release | grep RELEASE | awk -F= '{for ( i=0; i<=NF; i ++)  if($i~/[0-9]+\.*[0-9]*/) ver=$i }; END {print ver}'`
				if [ -n "${versionToken}" ]; then
					os_major_version=`echo $versionToken |  sed -e 's/\.[0-9]*//g'`
				fi
		
		######### Oracle Linux #########
			elif [ -f "/etc/oracle-release" ]; then
				oracle_flavor="y"
				versionToken=`cat /etc/oracle-release | grep release | awk '{for ( i=0; i<=NF; i ++)  if($i~/[0-9]+\.*[0-9]*/) ver=$i }; END {print ver}'`
				if [ -n "${versionToken}" ]; then				
					os_major_version=`echo $versionToken |  sed -e 's/\.[0-9]*//g'`
				fi
		######### New SuSE versions #########
      elif [ -f "/etc/os-release" ]; then
        cat /etc/os-release | grep "SUSE Linux" > /dev/null 2>&1
      	if [ $? -eq 0 ]; then
				  suse_flavor="y"
  				versionToken=`cat /etc/os-release | grep VERSION | awk '{for ( i=0; i<=NF; i ++)  if($i~/[0-9]+\.*[0-9]*/) ver=$i }; END {print ver}'`
  				if [ -n "${versionToken}" ]; then				
  					os_major_version=`cat /etc/os-release | grep "VERSION=" | awk -F "=" '{for ( i=0; i<=NF; i ++)  if($i~/[0-9]+\.*[0-9]*/) ver=$i }; END {print ver}'`
  				fi
      	fi
				
			else
				echo Unable to determine the Linux flavor.
				exit 1
			fi
		;;

		######### Oracle Solaris #########
		SunOS)
			os_major_version=10	# Setting default
			versionToken=`uname -r`
			if [ -n "${versionToken}" ]; then
				os_major_version=`echo $versionToken | sed -e 's/[0-9]\.//'`
			fi
		;;

		######### AIX #########
		AIX)
			os_major_version=6	# Setting default
            inst_fix="/usr/sbin/instfix"
			[ -x "${inst_fix}" ] && os_major_version=`${inst_fix} -it p | grep -i ml | tail -1 | sed "s/[a-zA-Z _]//g"`
			[ -z "${os_ver}"   ] && os_major_version=`oslevel`
			[ -z "${os_ver}"   ] && os_major_version=`uname -v`.`uname -r`
			os_major_version="`echo ${os_major_version} | cut -c1-3`"
		;;
		
		######### HPUX #########
		HP-UX)
			os_major_version=11.31 # Setting default
			versionToken=`uname -r`
			if [ -n "${versionToken}" ]; then
				os_major_version=`echo $versionToken | sed -e 's/[A-Z]\.//'`
			fi
		;;
		
		*)
			echo "Could not determine the operating system type (${machine_os_type})."
		;;
	esac
}

# return 32/64 bit
get_os_mode()
{
    [ "${generate_debug}" = "Y" ] && set -x
	
	is_mode_64_bit=0

	machine_os_type=`uname`	

	case ${machine_os_type} in
		linux|Linux)
			os_mode=`getconf LONG_BIT`
			;;
		SunOS)
			os_mode=`isainfo -b`
			;;
		AIX)
			os_mode=`getconf KERNEL_BITMODE`
			;;
		HP-UX)
			os_mode=`getconf KERNEL_BITS`
			;;
		*)
			echo "Could not determine the operating system type (${machine_os_type})."
			exit 1
		;;
	esac
	
	[ "${os_mode}" -eq 64 ] && is_mode_64_bit=1
}

check_linux_packs()
{
    [ "${generate_debug}" = "Y" ] && set -x
    packs_list=$*
    if [ -z "${packs_list}" ]; then
		return 0
    fi
    rc=0

	pkg_cmd="rpm -q" # default value
	
	[ "${ubuntu_flavor}" = "y" ] && pkg_cmd="dpkg -l"
	
	for pack in ${packs_list}
	do	
		${pkg_cmd} ${pack} > /dev/null  2>&1
		if [ $? -ne 0 ]; then
			echo "package ${pack} is required for successful JAVA run!"
			rc=1
		fi 
	done
    return $rc
}

check_aix_packs()
{
    [ "${generate_debug}" = "Y" ] && set -x
    packs_list=$*
    if [ -z "${packs_list}" ]; then
		return 0
    fi
    rc=0
    for pack in ${packs_list}
    do
	lslpp -l -q ${pack} > /dev/null  2>&1
	if [ $? -ne 0 ]; then
	    echo "fileset ${pack} is required for successful JAVA run!"
	    rc=1
	fi 
    done
    return $rc
}

check_solaris_packs()
{
	[ "${generate_debug}" = "Y" ] && set -x
	
	[ "${os_major_version}" -eq "10" ] && pkg_cmd="pkginfo"
	if [ "${os_major_version}" -ge "11" ]; then 
		pkg_cmd="pkg list"
		pkg version > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "\"pkg\" command, required for check_req run is not properly installed at your system!"
			echo "Exiting!"
			exit 1
		fi 
	fi
	
	packs_list=$*
	if [ -z "${packs_list}" ]; then
		return 0
	fi
	rc=0
	for pack in ${packs_list}
	do
		${pkg_cmd} ${pack} > /dev/null  2>&1
		if [ $? -ne 0 ]; then
			echo "package ${pack} is required for successful JAVA run!"
			rc=1
		fi 
	done
	return $rc
}

check_hpux_packs()
{
	[ "${generate_debug}" = "Y" ] && set -x
	
	packs_list=$*
    if [ -z "${packs_list}" ]; then
		return 0
    fi
    rc=0
    for pack in ${packs_list}
    do
		echo
		#swlist ....
	done
	return $rc
}

# check_Solaris11_packs()
# {
    # [ "${generate_debug}" = "Y" ] && set -x
    # packs_list=$*
    # if [ -z "${packs_list}" ]; then
		# return 0
    # fi
    # rc=0
    # for pack in ${packs_list}
    # do
		# pkg list ${pack} > /dev/null  2>&1
		# if [ $? -ne 0 ]; then
			# echo "package ${pack} is required for successful JAVA run!"
			# rc=1
		# fi 
    # done
    # return $rc
# }

CheckJavaRequirements()
{
    [ "${generate_debug}" = "Y" ] && set -x
    
	case ${machine_os_type} in
	linux|Linux)
	    if [ "${redhat_flavor}" = "y" ]; then		# Red Hat, CentOS and Amazon Linux
			[ ${os_major_version} = "5" ] && check_linux_packs "glibc" "glibc-common" "libtermcap"
			[ ${os_major_version} = "6" ] && check_linux_packs "glibc" "glibc-common" "ncurses-libs"
			[ ${os_major_version} = "7" ] && check_linux_packs "glibc" "glibc-common" "ncurses-libs"
			[ ${os_major_version} -ge 2015 ] && check_linux_packs "glibc" "glibc-common" "ncurses-libs" # for Amazon Linux
		fi
	    if [ "${suse_flavor}" = "y" ]; then
		if [ ${os_major_version} = "10" ]; then
			echo "SuSE 10 is not supported any more"
			exit 1
		fi
		if [ ${os_major_version} = "11" ]; then
			echo "SuSE 11 is not supported any more"
			exit 1
		fi
		check_linux_packs "glibc" "glibc-locale"
	    fi
		if [ "${ubuntu_flavor}" = "y" ]; then
				check_linux_packs "libc6:amd64" "libglib2.0-0:amd64" "libselinux1:amd64"
		fi
		if [ "${oracle_flavor}" = "y" ]; then
				check_linux_packs "glibc" "glibc-common" "ncurses-libs"
		fi
	    ;;
	SunOS)	
		if [ ${os_major_version} = "10" ]; then
			echo "Solaris 10 is not supported any more"
			exit 1
		fi
		[ "${os_major_version}" = "11" ] && check_solaris_packs "system/library" "system/locale" "system/library/math" "system/linker"
	    #[ "${os_major_version}" = "10" ] && check_solaris_packs "SUNWcslr" "SUNWcar" "SUNWnamos" "SUNWxwplt" "SUNWlibmsr"
	    ;;
	AIX)
	    check_aix_packs "bos.rte.libpthreads" "bos.rte.streams" "bos.rte.bind_cmds" "bos.rte.iconv"" bos.rte.loc" "bos.loc.iso.en_US" "bos.rte.security" "bos.rte.libc" "bos.rte.im" "bos.adt.lib"
	    ;;
	HP-UX)
		#check_hpux_packs ""
		;;
    esac
	
	return $rc
}

TestCorrectPlatform()
{
  return
    [ "${generate_debug}" = "Y" ] && set -x
	[ "${USER_SUPPLIED_JRE:-""}" = "Y" ] && return
    CorrectPlatformFlag="N"

    JavaDir=`dirname "${JAVA_LOCATION}"`

    case ${machine_os_type} in
	linux|Linux)
	    [ -f "${JavaDir}/../lib/amd64/libawt.so" ] && CorrectPlatformFlag="Y"
	    [ -f "${JavaDir}/../lib/ppc64le/libawt.so" ] && CorrectPlatformFlag="Y"
	    ;;
	SunOS)
		solaris_os_arc=`uname -p`
		case "${solaris_os_arc}" in
    		*sparc*) ;; #we support only sparc Solaris
    		*)
    			echo "Solaris on ${solaris_os_arc} is not supported"
    			exit 1
    			;;
		esac
	    [ -f "${JavaDir}/../lib"/sparc*/libawt.so ] && CorrectPlatformFlag="Y"
	    [ -f "${JavaDir}/../lib/i386/libawt.so" ] && CorrectPlatformFlag="Y"
	    ;;
	AIX)
	    [ -f "${JavaDir}/../lib/ppc/libawt.so" ] && CorrectPlatformFlag="Y"
	    ;;
	HP-UX)
		[ -f "${JavaDir}/../lib/IA64W/libawt.so" ] && CorrectPlatformFlag="Y"
		;;
    esac
    if [ "${CorrectPlatformFlag}" != "Y" ]; then
			echo "You run check for wrong platform."
			echo "Current platform is ${machine_os_type}"
			exit 1
    fi
}

get_java_location()
{
	#JRE_PATH already given by user
	if [ "${USER_SUPPLIED_JRE}" = "Y" ]; then
		JAVA_LOCATION="${JAVA_LOCATION}/bin/java"
		return
	fi	

	if [ -n "${BMC_INST_JAVA_HOME:-""}" ];then 
		if [ ! -d "${BMC_INST_JAVA_HOME}" ]; then
			BMC_INST_JAVA_HOME=""
		else
			JAVA_LOCATION="${BMC_INST_JAVA_HOME}/bin/java"
		fi
	fi
 
	if [ -f "${JAVA_LOCATION}" ]; then
		JAVA_LOCATION=`fix_path "${JAVA_LOCATION}"`
		PATH="${JAVA_LOCATION}:$PATH"
		export PATH
	else
		echo "Cannot find Java. Set BMC_INST_JAVA_HOME and rerun."
		exit 1
	fi
}

started_args=$*
if [ -z "${started_args:-""}" ]; then
	usage
fi
# my absoulte path
my_abs_path=`fix_path "${0}"`
my_dir=`dirname "${my_abs_path}"`

PATH="${my_dir}/Setup_files:$PATH"
export PATH

#Save ulimit output for tests in JAVA run
ulimit.sh
if [ $? -ne 0 ]; then
    echo "failed to get user's ulimits"
    exit 1
fi

#Default values for parameters:
product="CTMS"
version="9.0.21.300"
memory_model="MEDIUM"
database="PostgreSQL"

handle_startup_args

get_os_mode		# Determine if this is 32bit or 64bit operating system

if [ $is_mode_64_bit -ne 1 -a $machine_os_type != "AIX" ]; then
	#echo "32 bit Linux platforms are not supported!"
	echo "$machine_is_type operating system with 32bit processor are unsupported. Please install on 64bit machine."
	exit 1
fi
if [ $machine_os_type = "SunOS" ]; then
	solaris_os_arc=`uname -p`
	case "${solaris_os_arc}" in
		*sparc*) ;; #we support only sparc Solaris
		*)
			echo "Solaris on ${solaris_os_arc} is not supported"
			exit 1
			;;
	esac
fi

if [ $machine_os_type = "AIX" -a $product != "AGENT" -a $version = "9.0.21.300" ]; then
	echo "$machine_os_type operating system is unsupported for $product on version $version."
	exit 1
fi

get_os_version	# Determine the operating system version

get_java_location # Determine the JRE location

TestCorrectPlatform # Check that libraries are of the same platform (libawt)

DefsXmlFilePath="${my_dir}/Setup_files/XML_Definitions${machine_os_type}.xml"
if [ "${product:-""}" = "AGENT" ]; then
	database="NONE"
	case ${machine_os_type} in
	AIX|linux|Linux)
		if [ -f "${my_dir}/Setup_files/XML_Definitions${machine_os_type}Agent.xml" ]; then
			DefsXmlFilePath="${my_dir}/Setup_files/XML_Definitions${machine_os_type}Agent.xml"
		fi
	    ;;
    esac
fi


CheckJavaRequirements # Checks the Java requirements for running check_req
if [ $? -ne 0 ]; then
	echo "Failure to run JAVA runtime. Please check JAVA requirements"
	exit 1
fi

"${JAVA_LOCATION}" -version > /dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "Could not determine JRE version from ${JAVA_LOCATION}. Please make sure to point to the correct JRE"
	exit 1
fi

#java -Djava.util.logging.config.file=./logging.properties -Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,server=y,suspend=y,address=5005 -jar CheckReq.jar CTMS 9.0.00 PostgreSQL MEDIUM ./XML_DefinitionsRedHat_6.xml ./InternalTestDefinitions.xml

"${JAVA_LOCATION}" -Djava.util.logging.config.file=./logging.properties  -cp "${my_dir}/Setup_files/CheckReq.jar" com.bmc.CheckReq.MainFlow.Main ${product} ${version} ${database} ${memory_model} "${DefsXmlFilePath}" "${my_dir}/Setup_files/InternalTestDefinitions.xml"
exit 0
